import { createStore } from 'vuex'

export default createStore({
  state: {
    counter: 1,
    projects: {},
    user: null
  },

  mutations: {
    increment(state, amount) {
      state.counter += amount
    },
    decrement(state, amount) {
      state.counter -= amount
    },
    setProjects(state, projects) {
      state.projects = projects
    },
    setUser(state, user) {
      state.user = user
    }
  },
  getters: {
    isAuthenticated: state => !!state.user
  },
  actions: {
    async login({commit}, auth) {
      // Prepare headers & auth
      const headers = new Headers();
      headers.set('Content-Type', 'application/json');
      headers.set('Authorization', auth);

      // Prepare request
      const url = process.env.VUE_APP_API_URL + "users/";

      // Send response
      const response = await fetch(url, {
        method: 'GET',
        headers
      });
      if (!response.ok) {
        // Login failed
        commit('setUser', null)
      } else {
        // Get user object
        let response_json = await response.json();
        let user_object = response_json.results[0]
        user_object.auth = auth
        commit('setUser', user_object);
      }
    },
    async loadProjects({commit, state}) {
      // Prepare headers & auth
      const headers = new Headers();
      headers.set('Content-Type', 'application/json');
      headers.set('Authorization', state.user.auth);

      // Prepare request
      const url = process.env.VUE_APP_API_URL + "projects/";

      // Send response
      const response = await fetch(url, {
        method: 'GET',
        headers
      });
      if (!response.ok) {
        const message = `An error has occurred: ${response.status}`;
        throw new Error(message);
      }

      let response_json = await response.json();
      commit('setProjects', response_json.results);
    },
    async createProject({state, dispatch}, payload) {
      // Prepare headers & auth
      const headers = new Headers();
      headers.set('Content-Type', 'application/json');
      headers.set('Authorization', state.user.auth);

      // Prepare request
      const url = process.env.VUE_APP_API_URL + "projects/";

      // Send response
      const response = await fetch(url, {
        method: 'POST',
        headers,
        body: JSON.stringify(payload)
      });
      if (!response.ok) {
        const message = `An error has occurred: ${response.status}`;
        throw new Error(message);
      }
      // reload projects
      dispatch('loadProjects')
    },
    async deleteProject({state, dispatch}, id) {
      // Prepare headers & auth
      const headers = new Headers();
      headers.set('Content-Type', 'application/json');
      headers.set('Authorization', state.user.auth);

      // Prepare request
      const url = process.env.VUE_APP_API_URL + `projects/${id}/`;

      // Send response
      const response = await fetch(url, {
        method: 'DELETE',
        headers
      });
      if (!response.ok) {
        const message = `An error has occurred: ${response.status}`;
        throw new Error(message);
      }
      // reload projects
      dispatch('loadProjects')
    }
  },
  modules: {
  }
})
